# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model
from tools.translate import _
from openerp import netsvc


class gs_salesperson(Model):
    _name = 'gs.salesperson'
    _description = 'Salesperson'

    _columns = {
        'user_id': fields.many2one('res.users', 'Salesperson', required=True),
        'active': fields.boolean('Active'),
    }

    _defaults = {
        'active': True
    }

    _sql_constraints = [('user_unique', 'unique (user)', _('This User is configured!'))]

    def create(self, cr, uid, vals, context=None):
        context = context or {}
        margin_invoice_report_obj = self.pool.get('margin.invoice.report')
        period_obj = self.pool.get('account.period')
        salesperson_id = super(gs_salesperson, self).create(cr, uid, vals, context)
        user_id = self.browse(cr, uid, salesperson_id, context=context).user_id.id
        period_list = period_obj.search(cr, uid, [('special', '=', False)])
        if period_list:
            for period in period_list:
                vals_margin = {
                    'period_id': period,
                    'user_id': user_id
                }
                margin_invoice_report_obj.create(cr, uid, vals_margin, context)
        return salesperson_id

    def write(self, cr, uid, ids, vals, context=None):
        context = context or {}
        margin_invoice_report_obj = self.pool.get('margin.invoice.report')
        delete_list = []
        create_list = []
        old_instance = self.browse(cr, uid, ids[0])
        margin_ir_list_old = margin_invoice_report_obj.search(cr, uid, [('user_id', '=', old_instance.user_id.id)])
        if vals.get('active', False) and vals.get('active', False) == True:
            create_list.append(old_instance.user_id.id)
        elif vals.get('active', False) == False:
            for x in margin_ir_list_old:
                delete_list.append(x)
        if vals.get('user_id', False):
            for x in margin_ir_list_old:
                delete_list.append(x)
            create_list.append(vals.get('user_id', False))

        if create_list:
            period_obj = self.pool.get('account.period')
            period_list = period_obj.search(cr, uid, [('special', '=', False)])
            if period_list:
                for period in period_list:
                    for salesperson in create_list:
                        vals_margin = {
                            'period_id': period,
                            'user_id': salesperson
                        }
                        margin_invoice_report_obj.create(cr, uid, vals_margin, context)
        if delete_list:
            margin_invoice_report_obj.unlink(cr, uid, delete_list, context)
        return super(gs_salesperson, self).write(cr, uid, ids, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        context = context or {}
        margin_invoice_report_obj = self.pool.get('margin.invoice.report')
        margin_invoice_report_list = []
        for id in self.browse(cr, uid, ids):
            margin_invoice_report_list.append(margin_invoice_report_obj.search(cr, uid, [('user_id', '=', id.user_id.id)]))

        if margin_invoice_report_list:
            for list in margin_invoice_report_list:
                margin_invoice_report_obj.unlink(cr, uid, list, context)
        return super(gs_salesperson, self).unlink(cr, uid, ids, context)