# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model

import openerp.addons.decimal_precision as dp


class sale_order(Model):
    _name = 'sale.order'
    _inherit = 'sale.order'

    def _get_total_base_currency(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = sum([x.price_base_currency for x in obj.order_line])
        return res

    _columns = {
        'total_base_currency': fields.function(_get_total_base_currency,
                string='Total (USD)',
                digits_compute=dp.get_precision('Account'),
                store={
                    'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line'], 10),
                })
    }


class sale_order_line(Model):
    _name = 'sale.order.line'
    _inherit = 'sale.order.line'

    def _get_price_base_currency(self, cr, uid, ids, fields, args, context=None):
        cr_obj = self.pool.get('res.currency')
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            actual_price = obj.price_unit * obj.product_uom_qty
            res[obj.id] = actual_price
            if obj.order_id:
                base_currency_id = obj.order_id.company_id.currency_id.id
                order_currency_id = obj.order_id.pricelist_id.currency_id.id
                if base_currency_id != order_currency_id:
                    context['date'] = obj.order_id.date_order
                    res[obj.id] = cr_obj.compute(cr, uid, order_currency_id, base_currency_id, actual_price, round=False, context=context)
        return res

    _columns = {
        'price_base_currency': fields.function(_get_price_base_currency,
                string='Subtotal',
                digits_compute=dp.get_precision('Account'),
                store={
                    'sale.order.line': (lambda self, cr, uid, ids, c={}: ids, ['price_unit', 'product_uom_qty'], 10),
                })
    }
