# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime as dt

from openerp.osv import fields
from openerp.osv.orm import Model
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT as DF


class product_car(Model):
    _name = 'product.car'
    _inherits = {'product.product': 'product_id'}
    _inherit = ['mail.thread']
    _columns = {
        'product_id': fields.many2one('product.product', 'Product',
                                      required=True, ondelete="cascade"),
        'car_name': fields.related('product_id', 'name', type='char',
                                   string='Name', size=128, select=True,
                                   store=True),
    }

    def price_get_partner(self, cr, uid, cls, to_search, params, context=None):
        option = self.pool.get('option.value')
        model = self.pool.get(cls)
        price = 0.0
        sdate = params['start_date']
        edate = params['end_date']
        if sdate and edate:
            dsdate = dt.datetime.strptime(sdate, DF)
            dedate = dt.datetime.strptime(edate, DF)
            days = (dedate - dsdate).days
            all_days = option.search(cr, uid,
                                     [('option_type_id.code', '=', 'dq')],
                                     context=context)
            days_id = False
            for d in option.browse(cr, uid, all_days, context):
                mind, maxd = d.code.split('-')
                if days >= int(mind) and days <= int(maxd):
                    days_id = d.id
                    break
            to_search.append(('days_id', '=', days_id))
            pp_ids = model.search(cr, uid, to_search, context=context)
            if pp_ids:
                pp = model.browse(cr, uid, pp_ids[0], context)
                price = pp.price * days
        return price

    _order = 'car_name asc'


class pricelist_partnerinfo(Model):
    _name = 'pricelist.partnerinfo'
    _inherit = 'pricelist.partnerinfo'
    _columns = {
        'days_id': fields.many2one('option.value', 'Days',
                               domain="[('option_type_id.code', '=', 'dq')]"),
        'transmission_id': fields.many2one('option.value', 'Transmission',
                               domain="[('option_type_id.code', '=', 'tm')]")
    }
