# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model


class margin_invoice_report(Model):
    _name = 'margin.invoice.report'

    def _get_amount(self, cr, uid, ids, fields, args, context=None):
        ai = self.pool.get('account.invoice')
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            to_search = [('period_id', '=', obj.period_id.id),
                         ('user_id', '=', obj.user_id.id),
                         ('state', 'not in', ['draft', 'cancel'])]
            inv_ids = ai.search(cr, uid, to_search, context=context)
            res[obj.id] = {}
            for f in fields:
                res[obj.id][f] = sum([ai.get_in_base_currency(cr, uid, x.amount_total, x, context) for x in ai.browse(cr, uid, inv_ids) if x.type == f])
        return res

    def _get_margin(self, cr, uid, ids, fields, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context):
            res[obj.id] = 0
            if (obj.out_invoice - obj.out_refund) != 0.0:
                res[obj.id] = 100 - ((obj.in_invoice - obj.in_refund) / (obj.out_invoice - obj.out_refund) * 100)
        return res

    _columns = {
        'user_id': fields.many2one('res.users', 'User'),
        'period_id': fields.many2one('account.period', 'Period'),
        'out_invoice': fields.function(_get_amount, method=True, type='float',
                                  string='Sale Price', multi='amount'),
        'out_refund': fields.function(_get_amount, method=True, type='float',
                                  string='Refund', multi='amount'),
        'in_invoice': fields.function(_get_amount, method=True, type='float',
                                  string='Cost Price', multi='amount'),
        'in_refund': fields.function(_get_amount, method=True, type='float',
                                  string='Refund', multi='amount'),
        'margin': fields.function(_get_margin, method=True, type='float',
                                  string='Margin')
    }

    def zoom_invoice(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context)
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        ir_model_data = self.pool.get('ir.model.data')
        try:
            view_id = ir_model_data.get_object_reference(cr, uid, 'travel_core', 'view_zoom_invoice_tree')[1]
        except ValueError:
            view_id = False
        return {
            'name': 'Invoices',
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'tree',
            'view_id': view_id,
            'res_model': 'account.invoice',
            'domain': [('period_id', '=', obj.period_id.id), ('user_id', '=', obj.user_id.id)],
            'context': {'group_by': ['currency_id', 'type']}
        }
