# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv, orm
import openerp.addons.decimal_precision as dp


class sale_order(osv.osv):
    _inherit = 'sale.order'

    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        return super(sale_order, self)._amount_all(cr, uid, ids, field_name, arg, context=None)

    def _get_order(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('sale.order.line').browse(cr, uid, ids, context=context):
            result[line.order_id.id] = True
        return result.keys()

    _columns = {
        'amount_untaxed': fields.function(_amount_all, digits_compute=dp.get_precision('Price subtotal'),
                                          string='Untaxed Amount',
                                          store={
                                              'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line'], 10),
                                              'sale.order.line': (
                                                  _get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'],
                                                  10),
                                          },
                                          multi='sums', help="The amount without tax.", track_visibility='always'),
        'amount_tax': fields.function(_amount_all, digits_compute=dp.get_precision('Price subtotal'), string='Taxes',
                                      store={
                                          'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line'], 10),
                                          'sale.order.line': (
                                              _get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
                                      },
                                      multi='sums', help="The tax amount."),
        'amount_total': fields.function(_amount_all, digits_compute=dp.get_precision('Price subtotal'), string='Total',
                                        store={
                                            'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line'], 10),
                                            'sale.order.line': (
                                                _get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'],
                                                10),
                                        },
                                        multi='sums', help="The total amount."),
    }


class sale_order_line(osv.osv):
    _inherit = 'sale.order.line'

    def _amount_line(self, cr, uid, ids, prop, unknow_none, unknow_dict):
        return super(sale_order_line, self)._amount_line(cr, uid, ids, prop, unknow_none, unknow_dict)

    def _get_price_base_currency(self, cr, uid, ids, fields, args, context=None):
        return super(sale_order_line, self)._get_price_base_currency(cr, uid, ids, fields, args, context=None)

    _columns = {
        'price_subtotal': fields.function(_amount_line, string='Subtotal',
                                          digits_compute=dp.get_precision('Price subtotal')),
        'price_base_currency': fields.function(_get_price_base_currency,
                                               string='Subtotal',
                                               digits_compute=dp.get_precision('Price subtotal'),
                                               store={
                                                   'sale.order.line': (lambda self, cr, uid, ids, c={}: ids,
                                                                       ['price_unit', 'product_uom_qty'], 10),
                                               }),
        'price_unit': fields.float('Unit Price', required=True, digits_compute=dp.get_precision('Price subtotal'),
                                   readonly=True, states={'draft': [('readonly', False)]}),
    }
