# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv, orm
from openerp.osv.orm import Model


class res_currency_month_rate(Model):
    _name = 'res.currency.month.rate'
    _order = 'date desc'

    _columns = {
        'date': fields.date('Date'),
        'month_rate': fields.float('Month Rate', digits=(12, 6)),
        'currency_id': fields.many2one('res.currency', 'Currency Reference', ondelete='cascade', select=True),
        'month': fields.selection([
            ('01', 'January'),
            ('02', 'February'),
            ('03', 'March'),
            ('04', 'April'),
            ('05', 'May'),
            ('06', 'June'),
            ('07', 'July'),
            ('08', 'August'),
            ('09', 'September'),
            ('10', 'October'),
            ('11', 'November'),
            ('12', 'December'),
        ], 'Type', select=True, change_default=True, track_visibility='always'),
        'year': fields.char('Year', size=4),
    }

    def onchange_date(self, cr, uid, ids, date=False, context=None):
        result = {}
        if context is None:
            context = {}
        ctx = context.copy()
        if date:
            result = {'value': {
                'month': date[5:7],
                'year': date[0:4],
            }
            }
        return result

    def create(self, cr, uid, vals, context=None):
        context = context or {}

        result_id = self.search(cr, uid,
                                [('month', '=', vals['month']),
                                 ('date', '=', vals['date']),
                                 ('month_rate', '=', vals['month_rate']),
                                 ('currency_id', '=', vals['currency_id']),
                                 ('year', '=', vals['year'])])
        if not result_id:
            return super(res_currency_month_rate, self).create(cr, uid, vals, context)


class res_currency(Model):
    _inherit = 'res.currency'

    _columns = {
        'res_currency_month_rate_ids': fields.one2many('res.currency.month.rate', 'currency_id', 'Month Rate Lines'),
    }

    def onchange_month_rate(self, cr, uid, ids, month_rate=False, context=None):
        result = {}
        if context is None:
            context = {}
        ctx = context.copy()
        if month_rate:
            currency_month_rate_obj = self.pool.get('res.currency.month.rate')

            date_update = fields.date.context_today(self, cr, uid, context=ctx)
            currency_obj = self.browse(cr, uid, ids)
            if currency_obj:
                currency_id = currency_obj[0].id

                result_id = currency_month_rate_obj.search(cr, uid,
                                                           [('month', '=', date_update[5:7]),
                                                            ('currency_id', '=', currency_id),
                                                            ('year', '=', date_update[0:4])])
                if result_id:
                    vals_currency_month_rate_update = {
                        'date': date_update,
                        'month_rate': 1 / month_rate,
                    }

                    currency_month_rate_obj.write(cr, uid, result_id, vals_currency_month_rate_update)

                else:
                    vals_currency_month_rate = {
                        'date': date_update,
                        'month_rate': 1 / month_rate,
                        'month': date_update[5:7],
                        'year': date_update[0:4],
                        'currency_id': currency_id
                    }

                    currency_month_rate_obj.create(cr, uid, vals_currency_month_rate, context=ctx)

                # journal = self.pool.get('account.journal').browse(cr, uid, journal_id, context=context)
                # currency_id = journal.currency and journal.currency.id or journal.company_id.currency_id.id
                # company_id = journal.company_id.id

        res_currency_month_rate_ids = currency_month_rate_obj.search(cr, uid,
                                                                     [
                                                                         ('currency_id', '=', currency_id),
                                                                     ])

        result = {'value': {
            'res_currency_month_rate_ids': res_currency_month_rate_ids,
        }
        }
        return result
