# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime as dt

from openerp.osv import fields, osv
from openerp.osv.orm import Model
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT as DF


class product_hotel(Model):
    _name = 'product.hotel'
    _inherits = {'product.product': 'product_id'}
    _inherit = ['mail.thread']

    _columns = {
        'product_id': fields.many2one('product.product', 'Product',
                                      required=True, ondelete="cascade"),
        'res_partner_id': fields.many2one('res.partner', 'Contact'),
        'chain_id': fields.many2one('res.partner', 'Chain'),
        'stars': fields.selection([('1', '1'),
                                   ('2', '2'),
                                   ('3', '3'),
                                   ('4', '4'),
                                   ('5', '5')], 'Stars'),
        # TODO: este campo se debe usar con el campo product_type del modelo
        # producto
        'type_of_hotel': fields.selection([('city', 'City'),
                                           ('nature', 'Nature'),
                                           ('beach', 'Beach')], 'Type'),
        'destination': fields.many2one('destination', 'Destination'),
        'hotel_name': fields.related('product_id', 'name', type='char',
                                     string='Name', size=128, select=True,
                                     store=True)
    }

    def price_get_partner(self, cr, uid, cls, to_search, params, context=None):
        model = self.pool.get(cls)
        price = 0.0
        if params.get('hotel_1_room_type_id', False):
            to_search.append(('room_type_id',
                              '=', params['hotel_1_room_type_id']))
        # TODO: quitar ligero cable
        if params.get('hotel_2_meal_plan_id', False):
            ovid = int(params['hotel_2_meal_plan_id'])
            code = 'AI' if ovid == 9 else 'CP'
            to_search.append(('meal_plan_id.code', '=', code))
        pp_ids = model.search(cr, uid, to_search, context=context)
        sdate = params['start_date']
        edate = params['end_date']
        if sdate and edate and pp_ids:
            dsdate = dt.datetime.strptime(sdate, DF)
            dedate = dt.datetime.strptime(edate, DF)
            days = (dedate - dsdate).days
            pp = model.browse(cr, uid, pp_ids[0], context)
            adults = params.get('adults', 0)
            if adults == 1:
                #if pp.simple == False:
                #    raise osv.except_osv('Error!',
                #                         "There is not price for 1 adult.")
                price += pp.simple
            elif adults == 2:
                price += pp.price * 2
            elif adults == 3:
                #if pp.triple == False:
                #    raise osv.except_osv('Error!',
                #                         "There is not price for 3 adults.")
                price += pp.triple * 3
            else:
                extra = self.get_percent_or_int(pp.price, pp.extra_adult)
                price += (pp.price * 2 + (adults - 2) * extra)
            if params.get('children', 0) == 1:
                #if pp.child == False:
                #    raise osv.except_osv('Error!',
                #                         "There is not price for 1 child.")
                price += self.get_percent_or_int(pp.price, pp.child)
            if params.get('children', 0) == 2:
                #if pp.second_child == False:
                #    raise osv.except_osv('Error!',
                #                         "There is not price for 2 children.")
                price += self.get_percent_or_int(pp.price, pp.child)
                price += self.get_percent_or_int(pp.price, pp.second_child)
            paxs = params.get('adults', 0) + params.get('children', 0)
            if int(params.get('hotel_2_meal_plan_id', 0)) == 8:
                price += pp.add_map * paxs
            if int(params.get('hotel_2_meal_plan_id', 0)) == 7:
                price += pp.add_ap * paxs
            price *= days
        return price

    def get_percent_or_int(self, adult_price, val):
        if val == 1:
            return 0.0
        elif val > 1:
            return val
        elif val < 1:
            return adult_price * val

    _order = 'hotel_name asc'


class pricelist_partnerinfo(Model):
    _name = 'pricelist.partnerinfo'
    _inherit = 'pricelist.partnerinfo'
    _columns = {
        'meal_plan_id': fields.many2one('option.value', 'Meal Plan',
                            domain="[('option_type_id.code', '=', 'mp')]"),
        'room_type_id': fields.many2one('option.value', 'Room Type',
                            domain="[('option_type_id.code', '=', 'rt')]"),
        'simple': fields.float('Simple'),
        'triple': fields.float('Triple'),
        'extra_adult': fields.float('Extra Adult'),
        'second_child': fields.float('Second Child'),
        'add_map': fields.float('MAP Supp'),
        'add_ap': fields.float('AP Supp')
    }
