# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv.orm import Model
from openerp import netsvc


class account_invoice(Model):
    _name = 'account.invoice'
    _inherit = 'account.invoice'

    def group_invoice_lines_by_supplier(self, cr, uid, invoice, context):
        sol_obj = self.pool.get('sale.order.line')
        lines_by_supplier = {}
        for line in invoice.invoice_line:
            to_search = [('order_id.name', '=', line.origin),
                         ('product_id', '=', line.product_id.id)]
            sol_ids = sol_obj.search(cr, uid, to_search, context=context)
            if sol_ids:
                if len(sol_ids) > 1:
                    cr.execute('select order_line_id from \
                                sale_order_line_invoice_rel where \
                                invoice_id = %s', (line.id,))
                    sol_ids = cr.fetchall()[0]
                order_line = sol_obj.browse(cr, uid, sol_ids[0], context)
                supplier = sol_obj.get_supplier(order_line)
                if supplier.name == invoice.company_id.name:
                    if order_line.product_type == 'program':
                        for dl in order_line.program_detail_ids:
                            data = {
                               'invoice_line': line,
                               'sale_line': order_line,
                               'detail_line': dl
                            }
                            self.update_lines_by_supplier(lines_by_supplier,
                                                          dl.supplier_id, data)
                else:
                    data = {'invoice_line': line, 'sale_line': order_line}
                    self.update_lines_by_supplier(lines_by_supplier,
                                                  supplier, data)
        return lines_by_supplier

    def generate_supplier_invoices(self, cr, uid, inv_id, context=None):
        invoice = self.browse(cr, uid, inv_id, context)
        company_id = invoice.company_id
        journal_id = self.get_purchase_journal(cr, uid, company_id.id, context)
        vals = {
            'type': 'in_invoice',
            'state': 'draft',
            'journal_id': journal_id,
            'date_invoice': invoice.date_invoice,
            'period_id': invoice.period_id.id,
            'user_id': invoice.user_id.id,
            'company_id': company_id.id,
            'origin': invoice.origin,
            'comment': 'Generated from customer invoice ' + (invoice.origin or '')
        }

        lines_by_supplier = self.group_invoice_lines_by_supplier(cr, uid,
                                                             invoice, context)
        # repair bug the change cost currency
        sale_order = self.pool.get('sale.order')
        pl_obj = self.pool.get('product.pricelist')

        for s, lines in lines_by_supplier.items():
            data = vals.copy()
            data.update({
                'partner_id': s.id,
                'account_id': s.property_account_payable.id,
                'invoice_line': [],
            })
            if lines[0]['sale_line'].cost_currency_id:
                currency_id = lines[0]['sale_line'].cost_currency_id.id
            else:
                # repair bug the change cost currency
                sale_order_obj = sale_order.browse(cr, uid, lines[0]['sale_line'].order_id.id, context=context)
                currency_id = pl_obj.browse(cr, uid, sale_order_obj.pricelist_id.id).currency_id.id
                # currency_id = company_id.currency_id.id
            data['currency_id'] = currency_id
            for l in lines:
                sl = l['sale_line']
                il = l['invoice_line']
                if l.get('detail_line', False):
                    dl = l['detail_line']
                    cost_price = self.get_cost_price_for_program_line(cr, uid,
                                                                sl, dl, context)
                    line_vals = {
                        'name': dl.travel_product_id.name,
                        'origin': il.invoice_id.number,
                        'product_id': dl.travel_product_id.product_id.id,
                        'account_id': dl.travel_product_id.product_id.property_account_expense.id,
                        'quantity': il.quantity,
                        # 'discount': il.discount,
                        'price_unit': cost_price
                    }
                else:
                    cost_price = self.get_cost_price(cr, uid, sl, currency_id, context)
                    line_vals = {
                        'name': il.product_id.name,
                        'origin': il.invoice_id.number,
                        'product_id': il.product_id.id,
                        'account_id': il.product_id.property_account_expense.id,
                        'quantity': il.quantity,
                        # 'discount': il.discount,
                        'price_unit': cost_price
                    }
                data['invoice_line'].append((0, 0, line_vals))
            inv_id = self.create(cr, uid, data, context)

    def get_cost_price_for_program_line(self, cr, uid, sline, dline,
                                        context=None):
        if dline.price > 0:
            return dline.price
        product_obj = self.pool.get('product.product')
        line_obj = self.pool.get('sale.order.line')
        params = {
            'start_date': dline.start_date,
            'end_date': dline.end_date,
            'adults': sline.adults,
            'children': sline.children
        }
        tl = self.pool.get('travel.line')
        context_params = tl.get_context_params(cr, uid, sline, context)
        params.update(context_params)
        sinfo = line_obj.get_supplier_info(cr, uid, dline, context)
        price = product_obj.price_get_partner(cr, uid,
                                        dline.travel_product_id.product_id.id,
                                        sinfo, params, context)
        return price
