# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields
from openerp.osv.orm import Model


class product_product(Model):
    _name = 'product.product'
    _inherit = 'product.product'

    def price_get_partner(self, cr, uid, pid, partnerinfo_id, params,
                          context=None):
        if context is None:
            context = {}
        price = 0.0
        if params.get('program_line_ids', False):
            # TODO: revisar este caso, puede que ho haga falta
            for pl in params['program_line_ids']:
                if pl[0] == 0:
                    xparams = params.copy()
                    xparams.update(pl[2])
                    model, pid = pl[2]['travel_product_id'].split(',')
                    product = self.pool.get(model).browse(cr, uid, int(pid),
                                                          context).product_id
                    # TODO: determinar el partnerinfo_id para este producto
                    price += self.price_get_partner_line(cr, uid, product,
                                                        partnerinfo_id, xparams,
                                                        context)
                elif pl[0] in [1, 4]:
                    pass
        else:
            product = self.pool.get('product.product').browse(cr, uid, pid,
                                                              context)
            price = self.price_get_partner_line(cr, uid, product,
                                                partnerinfo_id,
                                                params, context)
        return price

    def price_get_partner_line(self, cr, uid, product, partnerinfo_id, params,
                               context=None):
        sdate = False
        edate = False
        to_search = [('suppinfo_id', '=', partnerinfo_id)]
        cls = 'pricelist.partnerinfo'
        if params.get('start_date', False):
            sdate = params['start_date']
            to_search += [('start_date', '<=', sdate),
                          ('end_date', '>=', sdate)]
        if params.get('end_date', False):
            edate = params['end_date']
            to_search += [('start_date', '<=', edate),
                          ('end_date', '>=', edate)]
        categ = product.categ_id.name.lower()
        klas = self.pool.get('product.' + categ)
        price = getattr(klas, 'price_get_partner')(cr, uid, cls, to_search,
                                                   params, context)
        return price


class product_program(Model):
    _name = 'product.program'
    _inherits = {'product.product': 'product_id'}
    _inherit = ['mail.thread']

    _columns = {
        'product_id':
            fields.many2one('product.product', 'Product', required=True,
                            ondelete="cascade"),
        'program_name':
            fields.related('product_id', 'name', type='char', string='Name',
                           size=128, select=True, store=True),
        'program_line_ids':
            fields.one2many('product.program.line', 'product_program_id',
                            'Lines'),
    }

    def price_get_partner(self, cr, uid, cls, to_search, params, context=None):
        model = self.pool.get(cls)
        price = 0.0
        paxs = params.get('adults', 0) + params.get('children', 0)
        to_search += [('min_pax', '<=', paxs), ('max_pax', '>=', paxs)]
        if params.get('program_1_meal_plan_id', False):
            to_search.append(('meal_plan_id',
                              '=', params['program_1_meal_plan_id']))
        pp_ids = model.search(cr, uid, to_search, context=context)
        if pp_ids:
            pp = model.browse(cr, uid, pp_ids[0], context)
            # TODO: esto pincha para la cantidad de paxs en habitaciones dobles,
            # no tiene en cuenta las habitaciones simples ni triples
            price = pp.price * paxs
        return price

    _order = 'program_name asc'


class product_program_line(Model):
    _name = 'product.program.line'
    _inherits = {'travel.line': 'travel_line_id'}
    _columns = {
        'travel_line_id':
            fields.many2one('travel.line', 'Travel Line',
                            required=True, ondelete="cascade", select=True),
        'supplier_id':
            fields.many2one('res.partner', 'Supplier'),
        'product_program_id':
            fields.many2one('product.program', 'Program'),
        'days':
            fields.integer('Days'),
        'price':
            fields.float('Price'),
        'meal_plan_id':
            fields.many2one('option.value', 'Meal Plan',
                            domain="[('option_type_id.code', '=', 'mp')]"),
        'sequence':
            fields.integer('Sequence')
    }

    _defaults = {
        'sequence': 0
    }

    _order = 'sequence'

    def up_sequence(self, cr, uid, ids, context=None):
        sq = self.read(cr, uid, ids[0], ['sequence'], context)['sequence']
        return self.write(cr, uid, ids[0], {'sequence': sq + 1})

    def down_sequence(self, cr, uid, ids, context=None):
        res = True
        sq = self.read(cr, uid, ids[0], ['sequence'], context)['sequence']
        if sq > 0:
            res = self.write(cr, uid, ids[0], {'sequence': sq - 1})
        return res


class pricelist_partnerinfo(Model):
    _name = 'pricelist.partnerinfo'
    _inherit = 'pricelist.partnerinfo'
    _columns = {
        'min_pax':
            fields.integer('Min'),
        'max_pax':
            fields.integer('Max'),
        'simple':
            fields.float('Simple'),
        'triple':
            fields.float('Triple')
    }
