# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime as dt

from openerp.osv import fields
from openerp.osv.orm import Model
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT as DF


class sale_order_line(Model):
    _name = 'sale.order.line'
    _inherit = 'sale.order.line'

    _columns = {
        'program_1_meal_plan_id': fields.many2one('option.value', 'Meal Plan',
                            domain="[('option_type_id.code', '=', 'mp')]",
                            readonly=True,
                             states={'draft': [('readonly', False)],
                                     'cancel': [('readonly', False)]}),
        'program_detail_ids': fields.one2many('sale.order.line.detail',
                                              'sale_order_line_id',
                                              'Details')
    }

    def onchange_product(self, cr, uid, ids, product, start_date, context=None):
        res = {'value': {'product_type': False}}
        if product:
            pmodel, ptid = product.split(',')
            prod = self.pool.get(pmodel).browse(cr, uid, int(ptid), context)
            prod_type = pmodel.split('.')[1]
            res['value'] = {
                'product_id': prod.product_id.id,
                'product_type': prod_type
            }
            if prod_type == 'program' and prod.program_line_ids:
                lines = []
                start_date = dt.datetime.strptime(start_date, DF)
                end_date = False
                sequence = 0
                for l in prod.program_line_ids:
                    p = l.travel_product_id
                    if end_date and l.sequence > sequence:
                        start_date = (end_date + dt.timedelta(1))
                    end_date = (start_date + dt.timedelta(l.days))
                    d = {
                      'travel_product_id': p and ','.join([p._name, str(p.id)]) or False,
                      'start_date': dt.datetime.strftime(start_date, DF),
                      'end_date': dt.datetime.strftime(end_date, DF),
                      'supplier_id': l.supplier_id.id,
                      'price': l.price,
                      'description': l.travel_line_id.product_description
                    }
                    lines.append((0, 0, d))
                    sequence = l.sequence
                res['value']['program_detail_ids'] = lines
                res['value']['supplier_id'] = prod.company_id.partner_id.id
        return res


class sale_order_line_detail(Model):
    _name = 'sale.order.line.detail'
    _inherits = {'travel.line': 'travel_line_id'}
    _columns = {
        'travel_line_id': fields.many2one('travel.line', 'Travel Line',
                                          required=True, ondelete="cascade",
                                          select=True),
        'sale_order_line_id': fields.many2one('sale.order.line',
                                              'Sale Order Line'),
        'price': fields.float('Price'),
        'supplier_id': fields.many2one('res.partner', 'Supplier',
                                       domain="[('supplier', '=', True)]",
                                       required=True),
        'description': fields.char('Description', size=256)
    }
