# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010, 2013, 2014 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import tools
from openerp.osv import fields, osv
from openerp.osv.orm import Model
from openerp.tools import float_round
import openerp.addons.decimal_precision as dp
from openerp.tools.translate import _
from openerp import netsvc


class account_voucher(Model):
    _name = 'account.voucher'
    _inherit = 'account.voucher'

    _columns = {
        'amount_rappel': fields.float('Amount Rappel', digits_compute=dp.get_precision('Discount')),
        'journal_rappel_id': fields.many2one('account.journal', 'Journal Rappel', domain=[('type', '=', 'cash')]),
        'rappel': fields.boolean('Rappel', help="This Payment have a rappel."),
        'move_rappels_id': fields.many2one('account.move', 'Account Entry Rappels'),

    }

    def cancel_voucher(self, cr, uid, ids, context=None):
        reconcile_pool = self.pool.get('account.move.reconcile')
        move_pool = self.pool.get('account.move')

        for voucher in self.browse(cr, uid, ids, context=context):
            # refresh to make sure you don't unlink an already removed move
            voucher.refresh()
            recs = []
            for line in voucher.move_ids:
                if line.reconcile_id:
                    recs += [line.reconcile_id.id]
                if line.reconcile_partial_id:
                    recs += [line.reconcile_partial_id.id]

            reconcile_pool.unlink(cr, uid, recs)

            if voucher.move_id:
                move_pool.button_cancel(cr, uid, [voucher.move_id.id])
                move_pool.unlink(cr, uid, [voucher.move_id.id])
            if voucher.move_rappels_id:
                move_pool.button_cancel(cr, uid, [voucher.move_rappels_id.id])
                move_pool.unlink(cr, uid, [voucher.move_rappels_id.id])
        res = {
            'state': 'cancel',
            'move_id': False,
            'move_rappels_id': False,
        }
        self.write(cr, uid, ids, res)
        return True

    def move_line_rappel_create(self, cr, uid, voucher_id, move_id, amount_rappel, context=None):
        if context is None:
            context = {}
        move_line_obj = self.pool.get('account.move.line')
        currency_obj = self.pool.get('res.currency')
        partner_pool = self.pool.get('res.partner')
        journal_pool = self.pool.get('account.journal')
        company_pool = self.pool.get('res.company')
        invoice_pool = self.pool.get('account.invoice')

        date = self.read(cr, uid, voucher_id, ['date'], context=context)['date']
        ctx = context.copy()
        ctx.update({'date': date})
        voucher = self.pool.get('account.voucher').browse(cr, uid, voucher_id, context=ctx)
        voucher_currency = voucher.journal_rappel_id.currency.id or voucher.company_id.currency_id.id
        partner_obj = partner_pool.browse(cr, uid, voucher.partner_id.id, context=ctx)
        company_obj = company_pool.browse(cr, uid, voucher.company_id.id, context=ctx)
        partner_name = partner_obj.name
        journal = journal_pool.browse(cr, uid, voucher.journal_rappel_id.id, context=context)
        # rappel account
        partner_rappel = partner_obj.property_account_rappel.id
        company_rappel = company_obj.property_account_rappel.id
        company_currency = self._get_company_currency(cr, uid, voucher.id, context)
        amount = 0.0
        amount = currency_obj.compute(cr, uid, voucher_currency, company_currency, amount_rappel, context=context)

        if partner_rappel:
            account_id = partner_rappel
        elif company_rappel:
            account_id = company_rappel
        else:
            raise osv.except_osv(_('Configuration Error!'), _("Please configure the account of rappel on " \
                                                              "the '%s' partner or in your company for the " \
                                                              "case a configuration be general.") % (partner_name))
        # cr.execute('SELECT account_invoice.origin'\
        #             ' FROM'\
        #               ' public.account_invoice,'\
        #               ' public.account_move_line,'\
        #               ' (select reconcile_partial_id as p,'\
        #               ' reconcile_id as r'\
        #              ' FROM account_move_line WHERE move_id=%s '\
        #                ' and ((reconcile_partial_id is not null) or (reconcile_id is not null))) as prueba'\
        #             ' WHERE account_invoice.move_id = account_move_line.move_id'\
        #               ' and account_move_line.reconcile_partial_id = prueba.p'\
        #               ' or account_move_line.reconcile_id = prueba.r',
        #            (voucher.move_id.id,))
        # val = cr.fetchone()
        l = None
        invoice = None
        for x in voucher.move_ids:
            if x.reconcile_id or x.reconcile_partial_id:
                l = x

        if l:
            line = move_line_obj.search(cr, uid, [('reconcile_id', '=', l.reconcile_id.id), ('id', '!=', l.id)],
                                        context=context)
            if not line:
                move_line_obj.search(cr, uid,
                                     [('reconcile_partial_id', '=', l.reconcile_partial_id.id), ('id', '!=', l.id)],
                                     context=context)
            if line:
                line_record = move_line_obj.browse(cr, uid, line[0], context=context)
                invoice = invoice_pool.search(cr, uid, [('move_id', '=', line_record.move_id.id)], context=context)
                invoice = invoice_pool.browse(cr, uid, invoice[0], context=context)

        move_line_obj.create(cr, uid, {
            'name': _('Rappels ') + (invoice and invoice.origin or 'None'),
            'move_id': move_id,
            'account_id': journal.default_debit_account_id.id or journal.default_credit_account_id.id,
            'debit': 0.0,
            'credit': amount,
            'period_id': voucher.period_id.id or False,
            'journal_id': voucher.journal_rappel_id.id or False,
            'partner_id': voucher.partner_id.id,
            'currency_id': company_currency != voucher_currency and voucher_currency or False,
            'amount_currency': company_currency != voucher_currency and -1 * amount_rappel or 0.0,
            'date': voucher.date,
        })
        move_line_obj.create(cr, uid, {
            'name': _('Rappels ') + (invoice and invoice.origin or 'None'),
            'move_id': move_id,
            'account_id': account_id,
            'credit': 0.0,
            'debit': amount,
            'period_id': voucher.period_id.id or False,
            'journal_id': voucher.journal_rappel_id.id or False,
            'partner_id': voucher.partner_id.id,
            'currency_id': company_currency != voucher_currency and voucher_currency or False,
            'amount_currency': company_currency != voucher_currency and 1 * amount_rappel or 0.0,
            'date': voucher.date,
        })

        return True

    def action_move_line_create(self, cr, uid, ids, context=None):
        '''
        Confirm the vouchers given in ids and create the journal entries for each of them
        '''
        if context is None:
            context = {}
        move_pool = self.pool.get('account.move')
        move_line_pool = self.pool.get('account.move.line')
        for voucher in self.browse(cr, uid, ids, context=context):
            local_context = dict(context, force_company=voucher.journal_id.company_id.id)
            if voucher.move_id:
                continue
            company_currency = self._get_company_currency(cr, uid, voucher.id, context)
            current_currency = self._get_current_currency(cr, uid, voucher.id, context)
            # we select the context to use accordingly if it's a multicurrency case or not
            context = self._sel_context(cr, uid, voucher.id, context)
            # But for the operations made by _convert_amount, we always need to give the date in the context
            ctx = context.copy()
            ctx.update({'date': voucher.date})
            # Create the account move record.
            move_id = move_pool.create(cr, uid, self.account_move_get(cr, uid, voucher.id, context=context),
                                       context=context)
            # Get the name of the account_move just created
            name = move_pool.browse(cr, uid, move_id, context=context).name
            # Create the first line of the voucher
            move_line_id = move_line_pool.create(cr, uid, self.first_move_line_get(cr, uid, voucher.id, move_id,
                                                                                   company_currency, current_currency,
                                                                                   local_context), local_context)
            move_line_brw = move_line_pool.browse(cr, uid, move_line_id, context=context)
            line_total = move_line_brw.debit - move_line_brw.credit
            rec_list_ids = []
            if voucher.type == 'sale':
                line_total = line_total - self._convert_amount(cr, uid, voucher.tax_amount, voucher.id, context=ctx)
            elif voucher.type == 'purchase':
                line_total = line_total + self._convert_amount(cr, uid, voucher.tax_amount, voucher.id, context=ctx)
            # Create one move line per voucher line where amount is not 0.0
            line_total, rec_list_ids = self.voucher_move_line_create(cr, uid, voucher.id, line_total, move_id,
                                                                     company_currency, current_currency, context)

            # Create the writeoff line if needed
            ml_writeoff = self.writeoff_move_line_get(cr, uid, voucher.id, line_total, move_id, name, company_currency,
                                                      current_currency, local_context)
            if ml_writeoff:
                move_line_pool.create(cr, uid, ml_writeoff, local_context)
            # We post the voucher.
            self.write(cr, uid, [voucher.id], {
                'move_id': move_id,
                'state': 'posted',
                'number': name,
            })
            if voucher.journal_id.entry_posted:
                move_pool.post(cr, uid, [move_id], context={})
            # We automatically reconcile the account move lines.
            reconcile = False
            for rec_ids in rec_list_ids:
                if len(rec_ids) >= 2:
                    reconcile = move_line_pool.reconcile_partial(cr, uid, rec_ids,
                                                                 writeoff_acc_id=voucher.writeoff_acc_id.id,
                                                                 writeoff_period_id=voucher.period_id.id,
                                                                 writeoff_journal_id=voucher.journal_id.id)

            # Post the move
            move_pool.button_validate(cr, uid, [move_id], context=context)
            # create two line per rappel
            if voucher.rappel and voucher.amount_rappel > 0:
                seq_obj = self.pool.get('ir.sequence')
                move_name = seq_obj.next_by_id(cr, uid, voucher.journal_rappel_id.sequence_id.id, context=context)
                move_vals = {
                    'name': move_name,
                    'date': voucher.date,
                    'period_id': voucher.period_id.id or False,
                    'journal_id': voucher.journal_rappel_id.id,
                    'narration': _('Generated from Partner Payment ') + name,
                    'ref': name,
                }
                move_id = move_pool.create(cr, uid, move_vals, context=context)
                # create two line per rappel
                self.move_line_rappel_create(cr, uid, voucher.id, move_id, voucher.amount_rappel, context=context)
                move_pool.button_validate(cr, uid, [move_id], context=context)
                self.write(cr, uid, [voucher.id], {
                    'move_rappels_id': move_id,
                })

        return True
