# -*- coding: utf-8 -*-

from openerp.osv.orm import Model
from openerp.osv import fields, osv
from openerp.tools.translate import _


class dataIntegraction(Model):
    _name = 'data.integration'

    _columns = {
        'name': fields.char('Operation'),
        'description': fields.text('Additional Information'),
        'data_in': fields.text('Data In'),
        'data_out': fields.text('Data Out'),
        'state': fields.selection([
            ('done', 'Done'),
            ('undone', 'UnDone'),
            ('error', 'Error'),
        ], 'Status', select=True, readonly=True),
        'date_operation': fields.date('Date', readonly=True, select=True),
    }

    _defaults = {
        'date_operation': fields.date.context_today,
    }

    def model_search(self, cr, uid, model, field, value, concept=None, context=None):
        vals = {
            'name': 'model_search',
            'description': 'model_search',
            'data_in': 'model: ' + model + ', field: ' + field + ', value: ' + value + ', concept: ' + concept,
            'data_out': 'model_search',
            'state': 'done',
        }
        try:
            model_pool = self.pool.get(model)

            if concept and concept == 'product':
                obj_id = model_pool.search(cr, uid, [(field, '=', value)], context)
                vals['description'] = 'operation done with product concept'
            elif concept and concept == 'costumer':
                obj_id = model_pool.search(cr, uid, [(field, '=', value)], context)
                if not obj_id:
                    obj_id = [model_pool.create(cr, uid, {field: value, 'costumer': True})]
                    vals['description'] = 'operation done with costumer concept created'
                else:
                    vals['description'] = 'operation done with costumer concept'
            elif concept and concept == 'supplier':
                obj_id = model_pool.search(cr, uid, [(field, '=', value)], context)
                vals['description'] = 'operation done with supplier concept'
            else:
                obj_id = model_pool.search(cr, uid, [(field, '=', value)], context)
                vals['description'] = 'operation done'
            if not obj_id:
                vals['state'] = 'undone'
                vals['data_out'] = 'None'
                self.create(cr, uid, vals)
                raise osv.except_osv(_('Error!'), _("The search is not correct."))
            vals['state'] = 'done'
            vals['data_out'] = unicode(obj_id[0])
            self.create(cr, uid, vals)
            return obj_id[0]
        except ValueError, e:
            vals['description'] = e
            vals['state'] = 'error'
            vals['data_out'] = 'error'
            self.create(cr, uid, vals)

    def create_sale_order_xmlrpc(self, cr, uid, saleorder_vals, context=None):
        sale_order_pool = self.pool.get('sale.order')
        sale_order_line_pool = self.pool.get('sale.order.line')
        company_pool = self.pool.get('res.company')
        vals = {
            'name': 'create_sale_order_xmlrpc',
            'description': 'create_sale_order_xmlrpc',
            'data_in': 'saleorder_vals: ' + unicode(saleorder_vals),
            'data_out': 'create_sale_order_xmlrpc',
            'state': 'done',
        }
        try:
            company_id = company_pool._company_default_get(cr, uid, 'sale.order', context)
            company_obj = company_pool.browse(cr, uid, company_id, context)
            new_saleorder = {}
            if not saleorder_vals:
                raise osv.except_osv(_('Error!'), _("The info not have data"))
            new_saleorder['partner_id'] = self.model_search(cr, uid, 'res.partner', 'name',
                                                            saleorder_vals['costumer_name'], 'costumer')
            new_saleorder['partner_shipping_id'] = new_saleorder['partner_id']
            new_saleorder['partner_invoice_id'] = new_saleorder['partner_id']
            new_saleorder['pricelist_id'] = company_obj.pricelist_id or 1
            new_saleorder['shop_id'] = company_obj.shop_id or 1
            new_saleorder['end_date'] = saleorder_vals['end_date']
            new_saleorder['date_order'] = saleorder_vals['date_order']

            sale_order_id = sale_order_pool.create(cr, uid, new_saleorder)

            new_orderline = []
            for line in saleorder_vals['order_line']:
                line_list = [0, False]
                model = line.keys()[0]
                name_value = line[model]['name']
                product_type_id = self.model_search(cr, uid, model, 'name', name_value, 'product')
                product_id = self.pool.get(model).browse(cr, uid, product_type_id, context=None).product_id.id

                program_detail_ids = line[model].get('program_detail_ids') or False
                program_list = []
                if program_detail_ids:
                    for program in program_detail_ids:
                        program_detail_list = [0, False]
                        program_model = program.keys()[0]
                        description_value = program[model]['description']
                        program_product_type_id = self.model_search(cr, uid, program_model, 'description',
                                                                    description_value, 'product')
                        program_val = {
                            'supplier_id': program[program_model].get('supplier_id') or False,
                            'description': program[program_model].get('description') or False,
                            'end_date': program[program_model].get('end_date') or False,
                            'price': program[program_model].get('price') or False,
                            'travel_product_id': unicode(program_model + ',' + unicode(program_product_type_id)) or False,
                            'start_date': program[program_model].get('start_date') or False
                        }
                        program_detail_list.append(program_val)
                        program_list.append(program_detail_list)
                program_detail = program_list

                line_val = {'order_id': sale_order_id,
                            'product_uos_qty': line[model].get('quantity’ ') or 1,
                            'flight_1_time': line[model].get('flight_1_time') or False,
                            'adults': line[model].get('adults') or 1,
                            'product_uom': line[model].get('product_uom') or 1,
                            'price_unit': line[model].get('price_unit') or 1,
                            'product_uom_qty': line[model].get('quantity') or 1,
                            'flight_2_time_back': line[model].get('flight_2_time_back') or False,
                            'car_4_return_place': line[model].get('car_4_return_place') or False,
                            'product_uos': line[model].get('product_uos') or False,
                            'children': line[model].get('children') or 0,
                            'price_unit_cost': line[model].get('price_unit_cost') or 1,
                            'car_2_pickup_place': line[model].get('car_2_pickup_place') or False,
                            'name': line[model].get('name') or False,
                            'transfer_4_return_time': line[model].get('transfer_4_return_time') or False,
                            'start_date': line[model].get('start_date') or False,
                            'description': line[model].get('description') or False,
                            'end_date': line[model].get('end_date') or False,
                            'discount': line[model].get('discount') or 0,
                            'transfer_5_return_place': line[model].get('transfer_5_return_place') or False,
                            'car_1_pickup_time': line[model].get('car_1_pickup_time') or False,
                            'reservation_number': line[model].get('reservation_number') or False,
                            'transfer_3_pickup_place': line[model].get('transfer_3_pickup_place') or False,
                            'car_3_return_time': line[model].get('car_3_return_time') or False,
                            'transfer_2_pickup_time': line[model].get('transfer_2_pickup_time') or False,
                            'hotel_1_room_type_id': line[model].get('hotel_1_room_type_id') or False,
                            'hotel_2_meal_plan_id': line[model].get('hotel_2_meal_plan_id') or False,
                            'cost_currency_id': line[model].get('cost_currency_id') or False,
                            'tax_id': [[6, False, []]],
                            'res_pax_ids': [[6, False, []]],
                            'supplier_id': line[model].get('supplier_id') or False,
                            'transfer_1_vehicle_type_id': False,
                            'program_detail_ids': program_detail or [],
                            'program_1_meal_plan_id': line[model].get('program_1_meal_plan_id') or False,
                            'product_id': product_id or False,
                            'travel_product_id': unicode(model + ',' + unicode(product_type_id)) or False
                            }

                sale_order_line_pool.create(cr, uid, line_val)

            sale_order_pool.action_button_confirm(cr, uid, [sale_order_id], context)
            sale_order_pool.action_invoice_create(cr, uid, [sale_order_id], False, None, False, context)
            vals['state'] = 'done'
            vals['data_out'] = unicode(True)
            self.create(cr, uid, vals)
            return True
        except ValueError, e:
            vals['description'] = e
            vals['state'] = 'error'
            vals['data_out'] = 'error'
            self.create(cr, uid, vals)
